from galaxy.exceptions import error_codes


def api_error_to_dict(**kwds):
    UNKNOWN_ERROR_CODE = error_codes.error_codes_by_name["UNKNOWN"]
    exception = kwds.get("exception", None)
    if exception:
        # If we are passed a MessageException use err_msg.
        default_error_code = getattr(exception, "err_code", UNKNOWN_ERROR_CODE)
        default_error_message = getattr(exception, "err_msg", default_error_code.default_error_message)
        extra_error_info = getattr(exception, "extra_error_info", {})
        if not isinstance(extra_error_info, dict):
            extra_error_info = {}
    else:
        default_error_message = "Error processing API request."
        default_error_code = UNKNOWN_ERROR_CODE
        extra_error_info = {}
    traceback_string = kwds.get("traceback", "No traceback available.")
    err_msg = kwds.get("err_msg", default_error_message)
    error_code_object = kwds.get("err_code", default_error_code)
    try:
        error_code = error_code_object.code
    except AttributeError:
        # Some sort of bad error code sent in, logic failure on part of
        # Galaxy developer.
        error_code = UNKNOWN_ERROR_CODE.code
    # Would prefer the terminology of error_code and error_message, but
    # err_msg used a good number of places already. Might as well not change
    # it?
    error_response = dict(err_msg=err_msg, err_code=error_code, **extra_error_info)
    if kwds.get("debug"):  # TODO: Should admins get to see traceback as well?
        error_response["traceback"] = traceback_string
    return error_response



"""Provides utilities for working with image files."""

import imghdr
import logging
from typing import (
    List,
    Optional,
)

try:
    from PIL import Image
except ImportError:
    Image = None  # type: ignore[assignment]

log = logging.getLogger(__name__)


def image_type(filename: str) -> Optional[str]:
    fmt = None
    if Image is not None:
        try:
            with Image.open(filename) as im:
                fmt = im.format
        except Exception:
            # We continue to try with imghdr, so this is a rare case of an
            # exception we expect to happen frequently, so we're not logging
            pass
    if not fmt:
        fmt = imghdr.what(filename)
    if fmt:
        return fmt.upper()
    else:
        return None


def check_image_type(filename: str, types: List[str]) -> bool:
    fmt = image_type(filename)
    if fmt in types:
        return True
    return False



"""Provide a consistent interface into and utilities for importlib file resources.
"""

import sys

if sys.version_info >= (3, 9):
    from importlib.resources import (
        as_file,
        files,
        Package as Anchor,
    )

    if sys.version_info >= (3, 12):
        from importlib.resources.abc import Traversable
    else:
        from importlib.abc import Traversable
else:
    from importlib_resources import (
        as_file,
        files,
        Package as Anchor,
    )
    from importlib_resources.abc import Traversable


def resource_path(package_or_requirement: Anchor, resource_name: str) -> Traversable:
    """
    Return specified resource as a Traversable.
    """
    return files(package_or_requirement).joinpath(resource_name)


def resource_string(package_or_requirement: Anchor, resource_name: str) -> str:
    """
    Return specified resource as a string.

    Replacement function for pkg_resources.resource_string, but returns unicode string instead of bytestring.
    """
    return resource_path(package_or_requirement, resource_name).read_text()


__all__ = (
    "as_file",
    "files",
    "resource_string",
    "resource_path",
    "Traversable",
)
